from hidori_core.modules.base import MODULES_REGISTRY
from hidori_core.modules.dnf import DnfModule
from hidori_core.modules.hello import HelloModule
from hidori_core.modules.hostname import HostnameModule

__all__ = ["MODULES_REGISTRY", "DnfModule", "HelloModule", "HostnameModule"]
